from pydantic import Field

from .base_models import BaseApiModel
from ..enums import UserRole


class User(BaseApiModel):
    name: str = Field(..., description="User's name")
    role: UserRole = Field(..., description="User's role")



from sqlalchemy.orm import Mapped, mapped_column

from .base_orm import BaseOrm
from ..enums import UserRole


class UserOrm(BaseOrm):
    __tablename__ = "users"

    name: Mapped[str] = mapped_column(index=True)
    role: Mapped[UserRole]



from typing import Optional

from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship

from .base_orm import BaseOrm
from .context_orm import TaskContextOrm
from .message_orm import TaskMessageOrm
from ..enums import ModelNameLLM, AiTaskStatus


class AiTaskOrm(BaseOrm):
    __tablename__ = "ai_tasks"

    llm_model_name: Mapped[ModelNameLLM]
    prompt_text: Mapped[str]
    status: Mapped[AiTaskStatus]
    user_prompt_id: Mapped[int] = mapped_column(ForeignKey("user_prompts.id"))
    scoped_task_id: Mapped[Optional[int]] = mapped_column(ForeignKey("scoped_tasks.id"))
    context: Mapped[Optional[TaskContextOrm]] = relationship(lazy="selectin")
    messages: Mapped[list[TaskMessageOrm]] = relationship(lazy="selectin")



from datetime import datetime, timezone
from typing import Optional

from pydantic import BaseModel, Field, ConfigDict


def utc_now() -> datetime:
    return datetime.now(timezone.utc)


class BaseApiModel(BaseModel):
    """
    Base class for all API models
    - Subclass of Pydantic BaseModel
    - Used for data validation and serialization in API layer
    """

    id: Optional[int] = Field(default=None, description="Unique identifier for the model")
    created_at: datetime = Field(default_factory=utc_now, description="Timestamp of when the record was created")
    updated_at: Optional[datetime] = Field(default=None, description="Timestamp of when the record was last updated")

    model_config = ConfigDict(
        from_attributes=True,
        # enable validate_assignment if we want to validate whenever updating model attributes
        # validate_assignment=True,
    )



from typing import Optional

from .base_models import BaseApiModel
from ..enums import StatusType, AssigneeType, NecessityType, PriorityType, TaskType


class BaseProjectModel(BaseApiModel):
    description: str
    status: StatusType
    name: str


class Project(BaseProjectModel):
    pass


class Objective(BaseProjectModel):
    project_id: int


class KeyResult(BaseProjectModel):
    objective_id: int


class Requirement(BaseProjectModel):
    key_result_id: int
    necessity: NecessityType


class ScopedTask(BaseProjectModel):
    requirement_id: int
    assignee: Optional[AssigneeType] = None
    priority: Optional[PriorityType] = None
    task_type: TaskType = TaskType.UNDEFINED

    # Additional fields to provide more information
    # estimated_duration: Optional[str]
    # difficulty: Optional[str]

    # This can be used to link the task to Gitlab MR
    # merge_request_id: Optional[str] = None


class ScopedTaskStep(BaseApiModel):
    task_id: int
    action: str
    status: StatusType
    expected_result: Optional[str] = None
    # consider adding execution_log here



from typing import Optional

from sqlalchemy import ForeignKey, JSON
from sqlalchemy.orm import Mapped, mapped_column

from .base_orm import BaseOrm


class TaskContextOrm(BaseOrm):
    __tablename__ = "task_contexts"

    response_format: Mapped[Optional[dict]] = mapped_column(JSON)
    project_context: Mapped[Optional[dict]] = mapped_column(JSON)
    input_files: Mapped[list[dict]] = mapped_column(JSON)
    output_files: Mapped[list[str]] = mapped_column(JSON)
    ai_task_id: Mapped[Optional[int]] = mapped_column(ForeignKey("ai_tasks.id"))
    scoped_task_id: Mapped[Optional[int]] = mapped_column(ForeignKey("scoped_tasks.id"))



from typing import Optional

from .base_models import BaseApiModel
from ..enums import MessageRole


# TODO: consider merging this module into ai_task_models.py


class TaskMessage(BaseApiModel):
    content: str
    role: MessageRole
    ai_task_id: Optional[int] = None



from typing import Optional

from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column

from .base_orm import BaseOrm
from ..enums import StatusType, AssigneeType, NecessityType, PriorityType, TaskType


class BaseProjectOrm(BaseOrm):
    __abstract__ = True

    description: Mapped[str]
    status: Mapped[StatusType] = mapped_column(index=True)
    name: Mapped[str] = mapped_column(index=True)


class ProjectOrm(BaseProjectOrm):
    __tablename__ = "projects"


class ObjectiveOrm(BaseProjectOrm):
    __tablename__ = "objectives"

    project_id: Mapped[int] = mapped_column(ForeignKey("projects.id"))
    # project: Mapped["Project"] = relationship(back_populates="objectives")
    # key_results: Mapped[list["KeyResult"]] = relationship(back_populates="objective")


class KeyResultOrm(BaseProjectOrm):
    __tablename__ = "key_results"

    objective_id: Mapped[int] = mapped_column(ForeignKey("objectives.id"))
    # objective: Mapped["Objective"] = relationship(back_populates="key_results")
    # requirements: Mapped[list["Requirement"]] = relationship(back_populates="key_result")


class RequirementOrm(BaseProjectOrm):
    __tablename__ = "requirements"

    necessity: Mapped[NecessityType]
    key_result_id: Mapped[int] = mapped_column(ForeignKey("key_results.id"))
    # key_result: Mapped["KeyResult"] = relationship(back_populates="requirements")
    # scoped_tasks: Mapped[list["ScopedTask"]] = relationship(back_populates="requirement")


class ScopedTaskOrm(BaseProjectOrm):
    __tablename__ = "scoped_tasks"

    assignee: Mapped[Optional[AssigneeType]]
    priority: Mapped[Optional[PriorityType]]
    task_type: Mapped[TaskType] = mapped_column(default=TaskType.UNDEFINED)
    requirement_id: Mapped[int] = mapped_column(ForeignKey("requirements.id"))
    # requirement: Mapped["Requirement"] = relationship(back_populates="scoped_tasks")
    # steps: Mapped[list["ScopedTaskStep"]] = relationship(back_populates="task")


class ScopedTaskStepOrm(BaseOrm):
    __tablename__ = "scoped_task_steps"

    action: Mapped[str]
    status: Mapped[StatusType]
    expected_result: Mapped[Optional[str]]
    task_id: Mapped[int] = mapped_column(ForeignKey("scoped_tasks.id"))
    # task: Mapped["ScopedTask"] = relationship(back_populates="steps")



from typing import Optional

from sqlalchemy import JSON, ForeignKey
from sqlalchemy.orm import Mapped, mapped_column

from .base_orm import BaseOrm
from ..enums import ModelNameLLM


class UserPromptOrm(BaseOrm):
    __tablename__ = "user_prompts"

    prompt_text: Mapped[str]
    response_class_name: Mapped[Optional[str]]
    input_files: Mapped[list[str]] = mapped_column(JSON)
    output_files: Mapped[list[str]] = mapped_column(JSON)
    llm_model_name: Mapped[ModelNameLLM]
    scoped_task_id: Mapped[Optional[int]] = mapped_column(ForeignKey("scoped_tasks.id"))



from typing import Optional

from .base_models import BaseApiModel
from .context_models import TaskContext
from .message_models import TaskMessage
from ..enums import ModelNameLLM, AiTaskStatus


class AiTask(BaseApiModel):
    llm_model_name: ModelNameLLM
    prompt_text: str
    status: AiTaskStatus
    user_prompt_id: int
    scoped_task_id: Optional[int] = None
    context: Optional[TaskContext] = None
    messages: list[TaskMessage] = []


class AiCompletion(BaseApiModel):
    success: bool
    text_output: Optional[str] = None
    json_output: Optional[dict] = None
    user_prompt_id: Optional[int] = None



from typing import Type

from ai_team_shared.models.api.ai_task_models import AiTask
from ai_team_shared.models.api.base_models import BaseApiModel
from ai_team_shared.models.api.context_models import TaskContext
from ai_team_shared.models.api.message_models import TaskMessage
from ai_team_shared.models.api.project_models import (
    Project,
    Objective,
    KeyResult,
    Requirement,
    ScopedTask,
    ScopedTaskStep,
)
from ai_team_shared.models.api.prompt_models import UserPrompt
from ai_team_shared.models.api.user_models import User
from ai_team_shared.models.orm.ai_task_orm import AiTaskOrm
from ai_team_shared.models.orm.base_orm import BaseOrm
from ai_team_shared.models.orm.context_orm import TaskContextOrm
from ai_team_shared.models.orm.message_orm import TaskMessageOrm
from ai_team_shared.models.orm.project_orm import (
    ProjectOrm,
    ObjectiveOrm,
    KeyResultOrm,
    RequirementOrm,
    ScopedTaskOrm,
    ScopedTaskStepOrm,
)
from ai_team_shared.models.orm.prompt_orm import UserPromptOrm
from ai_team_shared.models.orm.user_orm import UserOrm

# Register all models here
MODEL_REGISTRY: dict[Type[BaseApiModel], Type[BaseOrm]] = {
    User: UserOrm,
    Project: ProjectOrm,
    Objective: ObjectiveOrm,
    KeyResult: KeyResultOrm,
    Requirement: RequirementOrm,
    ScopedTask: ScopedTaskOrm,
    ScopedTaskStep: ScopedTaskStepOrm,
    UserPrompt: UserPromptOrm,
    AiTask: AiTaskOrm,
    TaskContext: TaskContextOrm,
    TaskMessage: TaskMessageOrm,
}



from typing import Optional

from pydantic import BaseModel

from .base_models import BaseApiModel
from .project_models import Project, Objective, KeyResult, Requirement, ScopedTask


# TODO: consider merging into ai_task_models.py


class TaskResponseFormat(BaseModel):
    data_model_class: str
    data_model_schema: Optional[dict] = None
    data_model_example: Optional[dict] = None


class ProjectContext(BaseModel):
    project: Project
    objective: Objective
    key_result: KeyResult
    requirement: Requirement
    scoped_task: ScopedTask


class FileContext(BaseModel):
    file_uri: str
    file_content: Optional[str] = None


class TaskContext(BaseApiModel):
    response_format: Optional[TaskResponseFormat] = None
    project_context: Optional[ProjectContext] = None
    input_files: list[FileContext] = []
    output_files: list[str] = []
    ai_task_id: Optional[int] = None
    scoped_task_id: Optional[int] = None



from typing import Optional

from .base_models import BaseApiModel
from ..enums import ModelNameLLM


class UserPrompt(BaseApiModel):
    prompt_text: str
    response_class_name: Optional[str] = None
    input_files: list[str] = []
    output_files: list[str] = []
    llm_model_name: ModelNameLLM
    scoped_task_id: Optional[int] = None

    # consider adding
    # - additional LLM model config fields (e.g., temperature)
    # - user_id field



from sqlalchemy import Column, Integer, DateTime, func
from sqlalchemy.orm import DeclarativeBase


class Base(DeclarativeBase):
    pass


class BaseOrm(Base):
    """
    Base class for all sqlalchemy ORM models
    - database schema
    - database CURD operations
    """

    __abstract__ = True

    id = Column(Integer, primary_key=True, autoincrement=True)
    created_at = Column(DateTime(timezone=True), server_default=func.now())
    updated_at = Column(DateTime(timezone=True), server_default=func.now(), onupdate=func.now())



from .project_models import (
    KeyResult,
    Objective,
    Requirement,
    ScopedTask,
    ScopedTaskStep,
)
from pydantic import BaseModel


# For specifying and validating the AI completion for project data types


class OKR(BaseModel):
    objective: Objective
    key_results: list[KeyResult]


class KeyResultRequirements(BaseModel):
    key_result: KeyResult
    requirements: list[Requirement]


class ScopedTaskStepList(BaseModel):
    task_id: int
    steps: list[ScopedTaskStep]


class RequirementTasks(BaseModel):
    requirement_id: int
    tasks: list[ScopedTask]



from typing import Optional

from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column

from .base_orm import BaseOrm
from ..enums import MessageRole


class TaskMessageOrm(BaseOrm):
    __tablename__ = "task_messages"

    content: Mapped[str]
    role: Mapped[MessageRole]
    ai_task_id: Mapped[Optional[int]] = mapped_column(ForeignKey("ai_tasks.id"))
